#include <Wire.h>
#include "Adafruit_SGP30.h"
#include <DHT.h>
#include <WiFi.h>
#include "ESPAsyncWebServer.h"

// Replace with your network credentials
const char* ssid     = "your SSID Wi-Fi lan";
const char* password = "your password";

#define DHTPIN 18
#define DHTTYPE DHT11   // DHT 11
//#define DHTTYPE    DHT22     // DHT 22 (AM2302)
//#define DHTTYPE    DHT21     // DHT 21 (AM2301)
DHT dht(DHTPIN, DHTTYPE);

Adafruit_SGP30 sgp;

/*uint32_t getAbsoluteHumidity(float temperature, float humidity) {
  // approximation formula from Sensirion SGP30 Driver Integration chapter 3.15
  const float absoluteHumidity = 216.7f * ((humidity / 100.0f) * 6.112f * exp((17.62f * temperature) / (243.12f + temperature)) / (273.15f + temperature)); // [g/m^3]
  const uint32_t absoluteHumidityScaled = static_cast<uint32_t>(1000.0f * absoluteHumidity); // [mg/m^3]
  return absoluteHumidityScaled;
  }*/
float tvoc;
float eco2;
float rawh2;
float rawethanol;
float temperature;
float temperaturef;
float humidity;
float t = 0.0;
float h = 0.0;

AsyncWebServer server(80);
AsyncEventSource events("/events");

unsigned long lastTime = 0;
unsigned long timerDelay = 10000;  // send readings timer

void getsgp30Readings() {

  temperature = t;
  temperaturef = (1.8 * t + 32);
  humidity = h;
  tvoc = sgp.TVOC;
  eco2 = sgp.eCO2;
  rawh2 = sgp.rawH2;
  rawethanol = sgp.rawEthanol;
}

String processor(const String& var) {
  getsgp30Readings();
  //Serial.println(var);
  if (var == "TEMPERATURE") {
    return String(temperature);
  }
  else if (var == "TEMPERATUREF") {
    return String(temperaturef);
  }
  else if (var == "HUMIDITY") {
    return String(humidity);
  }
  else if (var == "TVOC") {
    return String(tvoc);
  }
  else if (var == "eCO2") {
    return String(eco2);
  }
  else if (var == "raw_H2") {
    return String(rawh2);
  }
  else if (var == "raw_Ethanol") {
    return String(rawethanol);
  }
}

const char index_html[] PROGMEM = R"rawliteral(
<!DOCTYPE HTML><html>
<head>
  <title>SGP30 - DHT11  Weather Station</title>
  <meta name="viewport" content="width=device-width, initial-scale=1">
  <link rel="stylesheet" href="https://use.fontawesome.com/releases/v5.7.2/css/all.css" integrity="sha384-fnmOCqbTlWIlj8LyTjo7mOUStjsKC4pOpQbqyi7RrhN7udi9RwhKkMHpvLbHG9Sr" crossorigin="anonymous">
  <link rel="icon" href="data:,">
  <style>
    html {font-family: Arial; display: inline-block; text-align: center;}
    p {  font-size: 1.2rem;}
    body {  margin: 0;}
    .topnav { overflow: hidden; background-color: #4B1D3F; color: white; font-size: 1.7rem; }
    .content { padding: 20px; }
    .card { background-color: white; box-shadow: 2px 2px 12px 1px rgba(140,140,140,.5); }
    .cards { max-width: 700px; margin: 0 auto; display: grid; grid-gap: 2rem; grid-template-columns: repeat(auto-fit, minmax(300px, 1fr)); }
    .reading { font-size: 2.8rem; }
    .card.temperature { color: #0e7c7b; }
    .card.temperaturef { color: #101edd; }    
    .card.humidity { color: #17bebb; }
    .card.tvoc { color: #3fca6b; }
    .card.eco2 { color: #000000; }
    .card.rawh2 { color: #d62246; }
    .card.rawethanol { color: #e70df1; }    
  </style>
</head>
<body>
  <div class="topnav">
    <h3>SGP30 - DHT11  Weather Station</h3>
  </div>
  <div class="content">
    <div class="cards">
      <div class="card temperature">
        <h4><i class="fas fa-thermometer-half"></i> TEMPERATURE</h4><p><span class="reading"><span id="temp">%TEMPERATURE%</span> &deg;C</span></p>
      </div>
      <div class="card temperaturef">
        <h4><i class="fas fa-thermometer-half"></i> TEMPERATURE</h4><p><span class="reading"><span id="tempf">%TEMPERATUREF%</span> &deg;F</span></p>
      </div>
      <div class="card humidity">
        <h4><i class="fas fa-tint"></i> HUMIDITY</h4><p><span class="reading"><span id="hum">%HUMIDITY%</span> &percnt;</span></p>
      </div>
      <div class="card tvoc">
        <h4><i class="fas fa-angle-double-down"></i> TVOC</h4><p><span class="reading"><span id="tvocn">%TVOC%</span> ppb</span></p>
      </div>
      <div class="card eco2">
        <h4><i class="fas fa-wind"></i> eCO2</h4><p><span class="reading"><span id="eco2n">%eCO2%</span> ppm;</span></p>
      </div>
      <div class="card rawh2">
        <h4><i class="fas fa-solid fa-image"></i> raw_H2</h4><p><span class="reading"><span id="rawh2n">%RAW_H2%</span> ppm</span></p>
      </div>
       <div class="card rawethanol">
        <h4><i class="fas fa-solid fa-image"></i> raw_Ethanol</h4><p><span class="reading"><span id="rawethanoln">%RAW_Ethanol%</span> ppm</span></p>
      </div>
    </div>
  </div>
<script>
if (!!window.EventSource) {
 var source = new EventSource('/events');

 source.addEventListener('open', function(e) {
  console.log("Events Connected");
 }, false);
 source.addEventListener('error', function(e) {
  if (e.target.readyState != EventSource.OPEN) {
    console.log("Events Disconnected");
  }
 }, false);

 source.addEventListener('message', function(e) {
  console.log("message", e.data);
 }, false);

 source.addEventListener('temperature', function(e) {
  console.log("temperature", e.data);
  document.getElementById("temp").innerHTML = e.data;
 }, false);

 source.addEventListener('temperaturef', function(e) {
  console.log("temperaturef", e.data);
  document.getElementById("tempf").innerHTML = e.data;
 }, false);

 source.addEventListener('humidity', function(e) {
  console.log("humidity", e.data);
  document.getElementById("hum").innerHTML = e.data;
 }, false);

 source.addEventListener('tvoc', function(e) {
  console.log("tvoc", e.data);
  document.getElementById("tvocn").innerHTML = e.data;
 }, false);

 source.addEventListener('eco2', function(e) {
  console.log("eco2", e.data);
  document.getElementById("eco2n").innerHTML = e.data;
 }, false);

 source.addEventListener('rawh2', function(e) {
  console.log("rawh2", e.data);
  document.getElementById("rawh2n").innerHTML = e.data;
 }, false);

 source.addEventListener('rawethanol', function(e) {
  console.log("rawethanol", e.data);
  document.getElementById("rawethanoln").innerHTML = e.data;
 }, false);
}
</script>
</body>
</html>)rawliteral";

void setup() {
  Serial.begin(115200);
  Wire.begin(27, 22);
  /*while (!Serial) {
    delay(10);  // Wait for serial console to open!
    }*/
  Serial.begin(115200);
  Wire.begin(27, 22);

  // Set the device as a Station and Soft Access Point simultaneously
  WiFi.mode(WIFI_AP_STA);

  // Set device as a Wi-Fi Station
  WiFi.begin(ssid, password);
  while (WiFi.status() != WL_CONNECTED) {
    delay(1000);
    Serial.println("Setting as a Wi-Fi Station..");
  }
  Serial.print("Station IP Address: ");
  Serial.println(WiFi.localIP());
  Serial.println();

  dht.begin();
  Serial.println("SGP30 test");

  if (! sgp.begin()) {
    Serial.println("Sensor not found :(");
    while (1);
  }
  Serial.print("Found SGP30 serial #");
  Serial.print(sgp.serialnumber[0], HEX);
  Serial.print(sgp.serialnumber[1], HEX);
  Serial.println(sgp.serialnumber[2], HEX);

  // If you have a baseline measurement from before you can assign it to start, to 'self-calibrate'
  //sgp.setIAQBaseline(0x8E68, 0x8F41);  // Will vary for each sensor!


  // Handle Web Server
  server.on("/", HTTP_GET, [](AsyncWebServerRequest * request) {
    request->send_P(200, "text/html", index_html, processor);
  });

  // Handle Web Server Events
  events.onConnect([](AsyncEventSourceClient * client) {
    if (client->lastId()) {
      Serial.printf("Client reconnected! Last message ID that it got is: %u\n", client->lastId());
    }
    // send event with message "hello!", id current millis
    // and set reconnect delay to 1 second
    client->send("hello!", NULL, millis(), 10000);
  });
  server.addHandler(&events);
  server.begin();
}

int counter = 0;


void loop() {
  // If you have a temperature / humidity sensor, you can set the absolute humidity to enable the humditiy compensation for the air quality signals
  //float temperature = 22.1; // [°C]
  //float humidity = 45.2; // [%RH]
  //sgp.setHumidity(getAbsoluteHumidity(temperature, humidity));
  float h = dht.readHumidity();
  float t = dht.readTemperature(); // Read temperature as Celsius (the default)
  float f = dht.readTemperature(true); // Read temperature as Fahrenheit (isFahrenheit = true)

  if (isnan(h) || isnan(t) || isnan(f)) {
    Serial.println("Failed to read from DHT sensor!");
    return;
  }
  Serial.print( "Temperature (C)= " );
  Serial.println( t);
  Serial.print( " - Temperature (F)= " );
  Serial.println(f);
  Serial.print( " - Humidity = " );
  Serial.print( h);
  Serial.println( "%" );
  Serial.println( "");

  if (! sgp.IAQmeasure()) {
    Serial.println("Measurement failed");
    return;
  }

  Serial.print("TVOC "); Serial.print(sgp.TVOC); Serial.println(" ppb\t");
  Serial.print("eCO2 "); Serial.print(sgp.eCO2); Serial.println(" ppm");

  if (! sgp.IAQmeasureRaw()) {
    Serial.println("Raw Measurement failed");
    return;
  }
  Serial.print("Raw H2 "); Serial.print(sgp.rawH2); Serial.println(" \t");
  Serial.print("Raw Ethanol "); Serial.print(sgp.rawEthanol); Serial.println("");
  Serial.println(" ");

  // Send Events to the Web Server with the Sensor Readings
  events.send("ping", NULL, millis());
  events.send(String(t).c_str(), "temperature", millis());
  events.send(String(f).c_str(), "temperaturef", millis());
  events.send(String(h).c_str(), "humidity", millis());
  events.send(String(sgp.TVOC).c_str(), "tvoc", millis());
  events.send(String(sgp.eCO2).c_str(), "eco2", millis());
  events.send(String(sgp.rawH2).c_str(), "rawh2", millis());
  events.send(String(sgp.rawEthanol).c_str(), "rawethanol", millis());

  lastTime = millis();

  counter++;
  if (counter == 30) {
    counter = 0;

    uint16_t TVOC_base, eCO2_base;
    if (! sgp.getIAQBaseline(&eCO2_base, &TVOC_base)) {
      Serial.println("Failed to get baseline readings");
      return;
    }
    Serial.print("****Baseline values: eCO2: 0x"); Serial.print(eCO2_base, HEX);
    Serial.print(" & TVOC: 0x"); Serial.println(TVOC_base, HEX);
  }
  delay(10000);
}
